"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var parser_exports = {};
__export(parser_exports, {
  chooseValueParser: () => chooseValueParser,
  parseNodeAttributes: () => parseNodeAttributes,
  parseValue: () => parseValue
});
module.exports = __toCommonJS(parser_exports);
var import_constants = require("../constants");
var import_regex = require("../constants/regex");
var import_error = __toESM(require("../models/error"));
var import_cleaners = require("./cleaners");
var import_validators = require("./validators");
function chooseValueParser(parser = parseValue) {
  if (typeof parser === "string") {
    if (parser === "string") {
      return void 0;
    }
    return parseValue;
  }
  return parser;
}
function parseValue(value) {
  if (value.toLowerCase() === "true") return true;
  if (value.toLowerCase() === "false") return false;
  if (value.match(import_regex.INT_REGEX)) {
    return Number.parseInt(value);
  }
  if (value.match(import_regex.FLOAT_REGEX)) {
    return Number.parseFloat(value);
  }
  const date = Date.parse(value);
  return Number.isNaN(date) ? value : new Date(date);
}
function parseNodeAttributes(tag, tagname, sanitize, valueParser) {
  return tag.replace(new RegExp(`^<${tagname}`, "g"), "").replace(import_regex.CLOSING_TAG_REGEX, "").trim().split(import_regex.SPACES_REGEX).map((item) => {
    const splitedItem = item.trim().split("=").filter((item2) => !!item2.trim());
    if (splitedItem.length === 0) {
      return [];
    }
    if (splitedItem.length === 1) {
      throw new import_error.default(`Expected value after <${tagname} ${splitedItem[0]}=? >`);
    }
    if (!(0, import_validators.isValidAttributeName)(splitedItem[0])) {
      throw new import_error.default(`Invalid XML attribute name: "${splitedItem[0]}". Attribute names must start with a letter or underscore and can only contain letters, digits, hyphens, underscores, and periods.`);
    }
    return [splitedItem[0], splitedItem.slice(1).join("=")];
  }).filter((item) => item.length > 0).reduce((res, item) => {
    const attributeName = item[0].trim();
    if (attributeName) {
      const value = typeof item[1] === "string" ? parseAttributeValue(item[1], sanitize) : item[1];
      res[attributeName] = valueParser ? valueParser(value) : value;
    }
    return res;
  }, {});
}
function parseAttributeValue(value, sanitize) {
  value = value.trim();
  const firstChar = value[0];
  if (!import_constants.quotes.includes(firstChar)) {
    throw new import_error.default(`Invalid attribute value: Expected a quote (single or double) at the beginning, but found '${firstChar}' instead.`);
  }
  const lastChar = value[value.length - 1];
  if (lastChar !== firstChar) {
    throw new import_error.default(`Invalid attribute value: Expected a '${firstChar}' at the end, but found '${lastChar}' instead.`);
  }
  let innerValue = value.slice(1, value.length - 1);
  if (innerValue.includes(firstChar)) {
    throw new import_error.default(`Invalid attribute value: The character '${firstChar}' is used as a delimiter, but it appears inside the value. Please escape or remove the internal '${firstChar}' or replace it by ${import_constants.specialCharsMap[firstChar]}.`);
  }
  if (sanitize) {
    innerValue = (0, import_cleaners.sanitiseAttributeValue)(innerValue);
    const invalidSymbols = innerValue.match(import_regex.INVALID_SYMBOLS_REGEX);
    if (invalidSymbols && invalidSymbols.length > 0) {
      const firstInvalidSymbol = invalidSymbols[0];
      const equivalentSymbol = firstInvalidSymbol in import_constants.specialCharsMap ? ` by '${import_constants.specialCharsMap[firstInvalidSymbol]}'` : "";
      throw new import_error.default(`Invalid symbol found: '${firstInvalidSymbol}' in value '${innerValue}'. Please remove or replace it${equivalentSymbol}.`);
    }
  }
  return innerValue;
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  chooseValueParser,
  parseNodeAttributes,
  parseValue
});
//# sourceMappingURL=parser.js.map