"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var src_exports = {};
__export(src_exports, {
  JsonifyError: () => import_error.default,
  JsonifyResult: () => import_types.JsonifyResult,
  JsonifyXMLOptions: () => import_types.JsonifyXMLOptions,
  Node: () => import_types.Node,
  jsonify: () => jsonify
});
module.exports = __toCommonJS(src_exports);
var import_constants = require("./constants");
var import_regex = require("./constants/regex");
var import_cleaners = require("./helpers/cleaners");
var import_parser = require("./helpers/parser");
var import_validators = require("./helpers/validators");
var import_error = __toESM(require("./models/error"));
var import_types = require("./types");
var import_fs = __toESM(require("fs"));
function parseJson(xml, { parseAttributes = false, valueParser, sanitize = true } = {}) {
  xml = xml.replace(import_regex.XML_COMMENT_REGEX, "");
  const stack = [{
    attributes: {},
    nodes: {},
    tagname: "root"
  }];
  const matches = Array.from(xml.matchAll(import_regex.XML_DOC_REGEX));
  if (matches.length === 0) {
    throw new import_error.default("Invalid XML: No valid tags found in the document. The structure may be malformed or empty.");
  }
  const parser = (0, import_parser.chooseValueParser)(valueParser);
  const attributesParser = parseAttributes ? parser : void 0;
  matches.forEach((match, i) => {
    const tagname = match[1];
    const isClosing = !tagname;
    if (!(0, import_validators.isValidAttributeName)(tagname)) {
      throw new import_error.default(`Invalid XML tag name: "${tagname}". tag names must start with a letter or underscore and can only contain letters, digits, hyphens, underscores, and periods.`);
    }
    if (isClosing) {
      const openedTag = stack.pop();
      if (!openedTag) {
        throw new import_error.default(`Invalid XML: Found closing tag ${match[0]} without a matching opening tag.`);
      }
      if (!(0, import_validators.closesOpenedTag)(match[0], openedTag.tagname)) {
        throw new import_error.default(
          stack.length === 0 ? `Invalid XML: Unexpected closing tag ${match[0]} without a corresponding opening tag.` : `Invalid XML: Expected closing tag for <${openedTag.tagname} ...>, but found ${match[0]}.`
        );
      }
    } else {
      const parent = stack.at(stack.length - 1);
      const isSelfClosing = (0, import_validators.isSelfClosingTag)(match[0]);
      const node2 = { tagname };
      if (!isSelfClosing) {
        stack.push(node2);
      }
      const attributes = (0, import_parser.parseNodeAttributes)(match[0], match[1], sanitize, attributesParser);
      if (Object.keys(attributes).length > 0) {
        node2.attributes = attributes;
      }
      if (parent) {
        if (!parent.nodes) {
          parent.nodes = {};
        }
        if (!parent.nodes[tagname]) {
          parent.nodes[tagname] = node2;
        } else if (!Array.isArray(parent.nodes[tagname])) {
          parent.nodes[tagname] = [parent.nodes[tagname], node2];
        } else {
          parent.nodes[tagname].push(node2);
        }
      }
      if (i + 1 < matches.length && matches[i + 1][0].startsWith("</") && matches[i + 1][0].includes(match[1])) {
        const startIndex = match.index + match[0].length;
        const endIndex = matches[i + 1].index;
        let value = xml.slice(startIndex, endIndex);
        if (sanitize) {
          value = value ? (0, import_cleaners.sanitiseAttributeValue)(value) : value;
          const invalidSymbols = value.match(import_regex.INVALID_TAG_NAME_SYMBOLS_REGEX);
          if (invalidSymbols && invalidSymbols.length > 0) {
            const firstInvalidSymbol = invalidSymbols[0];
            const equivalentSymbol = firstInvalidSymbol in import_constants.specialCharsMap ? ` by '${import_constants.specialCharsMap[firstInvalidSymbol]}'` : "";
            throw new import_error.default(`Invalid symbol found: '${firstInvalidSymbol}' in value '${value}'. Please remove or replace it${equivalentSymbol}.`);
          }
        }
        value = value && parser ? parser(value) : value;
        node2.value = value;
      }
    }
  });
  if (stack.length > 1) {
    throw new import_error.default(`Invalid XML: Unclosed tag(s) detected. The tag <${stack[stack.length - 1].tagname}> is not closed.`);
  }
  if (stack.length === 0 || !stack[0].nodes) {
    throw new import_error.default("Invalid XML: The document was not fully processed, possible unclosed tags or malformed structure.");
  }
  const root = stack[0].nodes;
  const rootName = matches[0][1];
  if (Array.isArray(root[rootName])) {
    throw new import_error.default(`Invalid XML structure: Multiple root elements detected. An XML document must have a single root element, but found multiple instances of <${rootName}>.`);
  }
  const node = root[rootName];
  return [
    rootName,
    { [rootName]: node }
  ];
}
function jsonify(options) {
  if (typeof options === "string") {
    return parseJson(options);
  }
  let xml = "";
  if ("xml" in options) {
    xml = options.xml;
  } else if ("input" in options) {
    if (!import_fs.default.existsSync(options.input)) {
      throw new import_error.default(`File not found: The specified file at path "${options.input}" does not exist.`);
    }
    xml = import_fs.default.readFileSync(options.input, "utf-8");
  }
  const json = parseJson(xml, options);
  if (options.output) {
    import_fs.default.writeFileSync(options.output, JSON.stringify(json), "utf-8");
  }
  return json;
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  JsonifyError,
  JsonifyResult,
  JsonifyXMLOptions,
  Node,
  jsonify
});
//# sourceMappingURL=index.js.map